function countDecimals(value) {
    if (Math.floor(value) !== value)
        return value.toString().split(".")[1].length || 0;
    return 0;
}


sm.plot = {
    parse(plot) {
        if (typeof plot._container != "undefined") {
            plot._container.remove();
        }

        var mainSvg =  createSvgElement('svg');
        var contentG = createSvgElement('g');
        var viewSvg = createSvgElement('svg');
        var viewG = createSvgElement('g');
        var gridG = createSvgElement('g');
        var datasG = createSvgElement('g');
        var axisG = createSvgElement('g');

        var container = document.createElement('div');
        plot._container = container;
        container.classList.add('container');
        var yContainer = document.createElement('div');
        yContainer.classList.add('yContainer');
        var xContainer = document.createElement('div');
        xContainer.classList.add('xContainer');
        var plotContainer = document.createElement('div');
        plotContainer.classList.add('plotContainer');

        var images = plot.getElementsByTagName('img');
        while (images.length) {
            plotContainer.append(images[0]);
        }

        var xmin = parseFloat(plot.getAttribute('sm-xmin')); plot._xmin = xmin;
        var xmax = parseFloat(plot.getAttribute('sm-xmax')); plot._xmax = xmax;
        var ymin = parseFloat(plot.getAttribute('sm-ymin')); plot._ymin = ymin;
        var ymax = parseFloat(plot.getAttribute('sm-ymax')); plot._ymax = ymax; plot._oymax = plot._ymax;
        var xstep = parseFloat(plot.getAttribute('sm-xstep')); plot._xstep = xstep;
        var ystep = parseFloat(plot.getAttribute('sm-ystep')); plot._ystep = ystep; plot._oystep = plot._ystep;
        var ar = parseFloat(plot.getAttribute('sm-aspectratio')); plot._ar = ar;

        var xsteps = Math.ceil((xmax - xmin)/xstep);
        var ysteps = Math.ceil((ymax - ymin)/ystep);
        var xdecimals = countDecimals(xstep); plot._xdecimals = xdecimals;
        var ydecimals = countDecimals(ystep); plot._ydecimals = ydecimals;

        plot._xname = plot.getAttribute('sm-xname');
        plot._yname = plot.getAttribute('sm-yname');
        plot._xsymbol = plot.getAttribute('sm-xsymbol');
        plot._ysymbol = plot.getAttribute('sm-ysymbol');
        plot._xunit = plot.getAttribute('sm-xunit');
        plot._yunit = plot.getAttribute('sm-yunit');

        var width = 100;
        var height = 100*ar;

        viewSvg.classList.add('view');
        contentG.classList.add('content');
        mainSvg.classList.add('main');
        gridG.classList.add('grid');
        datasG.classList.add('datas');
        axisG.classList.add('axis');

        mainSvg.setAttribute('viewBox', '0 0 ' + width + ' ' + height);
        viewSvg.setAttribute('viewBox', '0 0 ' + width + ' ' + height);

        // Calculate factors:
        var yf = 100*ar/(ymax - ymin); plot._yf = yf; plot._oyf = plot._yf;
        var xf = 100/(xmax - xmin); plot._xf = xf;

        // Create grid and descriptions:
        var svgElement = undefined;
        var textElement = undefined;

        plot._yticks = [];
        for (var i = 1; i < ysteps; i++) {
            svgElement = createSvgElement('path');
            svgElement.setAttribute('d', 'M 0 ' + (ystep * i)*yf + ' L ' + (xmax - xmin)*xf + ' ' + (ystep * i)*yf);
            svgElement.classList.add('linear');
            gridG.append(svgElement);

            textElement = document.createElement('div');
            textElement.classList.add('tick');
            textElement.classList.add('linear');
            textElement.innerHTML = (ymin + ystep * i).toFixed(ydecimals).replace('.', ',');
            textElement.style.top = (1-(ystep * i)/(ymax - ymin))*100 + '%';
            yContainer.append(textElement);
            plot._yticks.push(textElement);
        }
        textElement = document.createElement('div');
        textElement.classList.add('name');
        textElement.innerHTML = plot._yname + ' ' + plot._ysymbol + ' [' + plot._yunit + ']';
        yContainer.append(textElement);

        plot._xticks = [];
        for (var i = 1; i < xsteps; i++) {
            svgElement = createSvgElement('path');
            svgElement.setAttribute('d', 'M ' + (i * xstep)*xf + ' 0 L ' + (i * xstep)*xf + ' ' + (ymax - ymin)*yf);
            gridG.append(svgElement);

            textElement = document.createElement('div');
            textElement.classList.add('tick');
            textElement.innerHTML = (xmin + xstep * i).toFixed(xdecimals).replace('.', ',');
            textElement.style.left = (xstep * i)/(xmax - xmin)*100 + '%';
            xContainer.append(textElement);
            plot._xticks.push(textElement);
        }
        textElement = document.createElement('div');
        textElement.classList.add('name');
        textElement.innerHTML = plot._xname + ' ' + plot._xsymbol + ' [' + plot._xunit + ']';
        xContainer.append(textElement);

        // Logarithmic Axis and Ticks:
        if (plot.hasAttribute('sm-log')) {
            plot._logmax = parseInt(plot.getAttribute('sm-logmax'));
            plot._logmin = parseInt(plot.getAttribute('sm-logmin'));
            plot._logf = 100*ar/(plot._logmax - plot._logmin);
            plot._logsteps = plot._logmax - plot._logmin;

            for (var i = 0; i < plot._logsteps; i++) {
                for (var j = 1; j <= 10; j++) {
                    var log = (i + (1 - Math.log10(j))) * plot._logf;
                    svgElement = createSvgElement('path');
                    svgElement.setAttribute('d', 'M 0 ' + log + ' L ' + (xmax - xmin)*xf + ' ' + log);
                    svgElement.classList.add('log');
                    gridG.append(svgElement);
                }
            }

            plot._logticks = [];
            for (var i = 0; i <= plot._logsteps; i++) {
                textElement = document.createElement('div');
                textElement.classList.add('tick');
                textElement.classList.add('log');
                textElement.innerHTML = (Math.pow(10, i + plot._logmin)).toString().replace('.',',');
                textElement.style.top = (1-i/(plot._logmax - plot._logmin))*100 + '%';
                yContainer.append(textElement);
                plot._logticks.push(textElement);
            }
        }

        // Create Axis:
        svgElement = createSvgElement('path');
        svgElement.classList.add('axis');
        svgElement.setAttribute('d', 'M 0 ' + (ymax - ymin)*yf + ' l ' + (xmax - xmin)*xf + ' 0 m -1 1 l 1 -1 l -1 -1');
        axisG.append(svgElement);

        svgElement = createSvgElement('path');
        svgElement.classList.add('axis');
        svgElement.setAttribute('d', 'M 0 ' + (ymax - ymin)*yf + ' L 0 0 m -1 1 l 1 -1 l 1 1');
        axisG.append(svgElement);

        // Add Elements to DOM:
        viewG.append(gridG);
        viewG.append(datasG);
        viewSvg.append(viewG);
        contentG.append(viewSvg);
        contentG.append(axisG);
        mainSvg.append(contentG);

        // Create info box:
        var infoBox = document.createElement('div');
        plot._infoBox = infoBox;
        infoBox.classList.add('infoBox');
        var xInfo = document.createElement('div');
        plot._xInfo = xInfo;
        xInfo.classList.add('x');
        infoBox.append(xInfo);
        var yInfo = document.createElement('div');
        plot._yInfo = yInfo;
        yInfo.classList.add('y');
        infoBox.append(yInfo);

        // Create cursor:
        var cursor = document.createElement('div');
        plot._cursor = cursor;
        cursor.classList.add('cursor');
        svgElement = createSvgElement('path');
        plot._xCursor = svgElement;
        svgElement.classList.add('cursor');
        viewSvg.append(svgElement);
        svgElement = createSvgElement('path');
        plot._yCursor = svgElement;
        svgElement.classList.add('cursor');
        viewSvg.append(svgElement);
        
        plotContainer.append(cursor);
        plotContainer.append(infoBox);

        plot._datasG = datasG;
        plot._datas = [];
        sm.plot.data.parseFrom(plot, plot.getElementsByTagName('sm-data'));
        
        plotContainer.append(mainSvg);
        container.append(yContainer);
        container.append(plotContainer);
        container.append(xContainer);
        plot.append(container);

        mainSvg._plot = plot;
        mainSvg.addEventListener('click', sm.plot.click);
        mainSvg.addEventListener('mousemove', sm.plot.mousemove);
    },

    convertX(plot, x) {
        return (x - plot._xmin) * plot._xf;
    },

    convertY(plot, y) {
        return ((plot._ymax - plot._ymin) - (y - plot._ymin)) * plot._yf;
    },

    convertLogY(plot, y) {
        if (y > 0) {
            return ((plot._logmax - plot._logmin) - (Math.log10(y) - plot._logmin)) * plot._logf;
        } else {
            return -100;
        }
    },

    getCursorX(plot, x) {
        return x;
    },

    getCursorY(plot, y) {
        return y;
    },

    scaleY(plot, factor) {
        plot._ymax = (plot._oymax - plot._ymin) * factor;
        plot._ystep = plot._oystep * factor;
        plot._yf = plot._oyf / factor;

        for (var i = 1; i <= plot._yticks.length; i++) {
            plot._yticks[i - 1].innerHTML = (plot._ymin + plot._ystep * i).toFixed(plot._ydecimals).replace('.', ',');
        }
        for (var i = 0; i < plot._datas.length; i++) {
            sm.plot.data.update(plot._datas[i]);
        }
    },

    click(event) {
        var infoBox = this._plot._infoBox;

        var dim = this.getBoundingClientRect();
        infoBox.style.left = (event.clientX - dim.left)/dim.width*100 + '%';
        infoBox.style.top = (event.clientY - dim.top)/dim.height*100 + '%';
    },

    resize() {
        console.log('tets');
        var dim = this.getBoundingClientRect();

        if (dim.width < 500) {
            this.classList.add('small');
        } else {
            this.classList.remove('small');
        }
    },

    mousemove(event) {
        var plot = this._plot;

        var dim = this.getBoundingClientRect();
        var xPercent = (event.clientX - dim.left)/dim.width*100,
            yPercent = (event.clientY - dim.top)/dim.height*100;
        var cursor = plot._cursor,
            infoBox = plot._infoBox;

        cursor.style.left = xPercent + '%';
        cursor.style.top = yPercent + '%';
        infoBox.style.left = xPercent + '%';
        infoBox.style.top = yPercent + '%';

        plot._xCursor.setAttribute('d', 'M ' + xPercent + ' 0 l 0 ' + (100*plot._ar));
        plot._yCursor.setAttribute('d', 'M 0 ' + yPercent*plot._ar + ' l 100 0');

        var xCoordinate = xPercent / plot._xf + plot._xmin,
            yCoordinate = (100 - yPercent)*plot._ar / plot._yf + plot._ymin;
        
        plot._xInfo.innerHTML = plot._xsymbol + ' = ' + xCoordinate.toFixed(plot._xdecimals).replace('.',',') + ' ' + plot._xunit;
        plot._yInfo.innerHTML = plot._ysymbol + ' = ' + yCoordinate.toFixed(plot._ydecimals).replace('.',',') + ' ' + plot._yunit;
    },

    update(plot) {
        for (var i = 0; i < plot._datas.length; i++) {
            sm.plot.data.update(plot._datas[i]);
        }
    },

    data: {
        parse(plot, data) {
            data._plot = plot;
            data._dataG = createSvgElement('g');
            plot._datas.push(data);

            if (data.hasAttribute('sm-array')) {
                data._points = eval(data.getAttribute('sm-array'));
            } else if (data.hasAttribute('sm-source')) {
                data._points = eval(data.getAttribute('sm-source'))[data.getAttribute('sm-name')];
            } else {
                data._points = [];
            }
            if (data.hasAttribute('sm-points')) {
                // Übertrage angegebene Punkte im Format "x1,y1;x2,y2;..." in ein Arrray:
                var strings = data.getAttribute('sm-points').split(';');
                for (var i = 0; i < strings.length; i++) {
                    data._points.push(strings[i].split(',').map(parseFloat));
                }
            }

            // Erstelle Pfad, falls eine Interpolation gefordert ist:
            if (data.hasAttribute('sm-interpolate')) {
                data._path = createSvgElement('path');
                data._dataG.append(data._path);
            }

            // Erstelle Punkte, falls Punkte gefordert sind:
            if (data.hasAttribute('sm-markers')) {
                data._markers = [];
                if (data._plot.hasAttribute('sm-log')) {
                    data._logmarkers = [];
                }
            }

            plot._datasG.append(data._dataG);

            sm.plot.data.update(data);
        },

        parseFrom(plot, datas) {
            for (var i = 0; i < datas.length; i++) {
                sm.plot.data.parse(plot, datas[i]);
            }
        },

        update(data) {
            var convertX = sm.plot.convertX;
            var convertY = sm.plot.convertY;
            var convertLogY = sm.plot.convertLogY;
            var plot = data._plot;
            var points = data._points;
            var xmax = 1000000;

            if (data.hasAttribute('sm-xmax')) {
                xmax = parseInt(data.getAttribute('sm-xmax'));
            }

            // Pfad updaten, falls vorhanden (bzw. falls "sm-interpolate" angegeben ist):
            if (data.hasAttribute('sm-interpolate')) {
                var d = 'M '
                for (var i = 0; i < points.length; i++) {
                    if (i > 0) {
                        d = d + ' L ';
                    }
                    d = d + convertX(plot, points[i][0]) + ' ' + convertY(plot, points[i][1]);
                }
                data._path.setAttribute('d', d);
            }
            if (data.hasAttribute('sm-markers')) {
                var start = data._markers.length;
                for (var i = start; i < data._points.length; i++) {
                    var marker = createSvgElement('circle');
                    marker.setAttribute('r', '1');
                    marker.classList.add('linear');
                    data._markers.push(marker);
                    data._dataG.append(marker);
                }

                for (var i = 0; i < points.length; i++) {
                    data._markers[i].setAttribute('cx', convertX(plot, points[i][0]));
                    data._markers[i].setAttribute('cy', convertY(plot, points[i][1]));
                    if (points[i][0] < xmax) {
                        data._markers[i].classList.remove('hidden');
                    } else {
                        data._markers[i].classList.add('hidden');
                    }
                }
                if (data._plot.hasAttribute('sm-log')) {
                    var start = data._logmarkers.length;
                    for (var i = start; i < data._points.length; i++) {
                        var logmarker = createSvgElement('circle');
                        logmarker.setAttribute('r', '1');
                        logmarker.classList.add('log');
                        data._logmarkers.push(logmarker);
                        data._dataG.append(logmarker);
                    }
                    for (var i = 0; i < points.length; i++) {
                        data._logmarkers[i].setAttribute('cx', convertX(plot, points[i][0]));
                        data._logmarkers[i].setAttribute('cy', convertLogY(plot, points[i][1]));
                    }
                }
            }
        }
    },

    parseFrom(elements) {
        for (var i = 0; i < elements.length; i++) {
            sm.plot.parse(elements[i]);
        }
    }
}